/*

*/

import java.util.concurrent.RunnableFuture;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.concurrent.ExecutionException;

class MyRuunableFuture implements RunnableFuture<Integer>
{
   public Integer result;
   Thread thread = null;
   boolean cancelled = false;

   public MyRuunableFuture()
   {
      this.thread = new Thread(this);
      thread.start();
   }

   public void run()
   {
      java.util.Random r = new java.util.Random();
      int time = 1 + r.nextInt(10);  // random int between 1 and 10
      try
      {
         Thread.sleep(time*1000); // sleep time seconds
      }
      catch (InterruptedException e){System.err.println(e);}
      result = time;
   }

   public Integer get()
   {
      // Waits if necessary for the computation to complete, and then retrieves its result.
      try
      {
         this.thread.join();
      }
      catch (InterruptedException e){System.out.println(e);}
      return this.result;
   }

   public Integer get(long timeout, TimeUnit unit) throws TimeoutException
   {
      // Waits if necessary for at most the given time for the computation
      // to complete, and then retrieves its result, if available.
      try
      {
         this.thread.join(timeout);
      }
      catch (InterruptedException e){System.err.println(e);}
      if ( thread.isAlive() )
      {
         throw new TimeoutException("future result not ready");
      }
      else
      {
         return this.result;
      }
   }

   public boolean isDone()
   {
      // Returns true if this task completed.
      return ! thread.isAlive();
   }

   public boolean cancel(boolean mayInterruptIfRunning)
   {
      // http://www.javapractices.com/topic/TopicAction.do?Id=251
      // https://www.ibm.com/developerworks/java/library/j-jtp05236/index.html

      // Attempt to cancel execution of this task.
      if ( isDone() || isCancelled() )
      {
         return false;
      }
      else
      {
         thread.interrupt();
         cancelled = true;
         return cancelled;
      }
   }

   public boolean isCancelled()
   {
      // Returns true if this task was canceled before it completed normally.

      return cancelled;
   }
}

public class Simple_RunnableFuture_Example_1
{
   public static void main(String[] args)
   {
      RunnableFuture<Integer> future = new MyRuunableFuture();
      int answer = -1;
      try
      {
         answer = future.get(); // block until the future is available
      }
      catch (ExecutionException e){System.err.println(e);}
      catch (InterruptedException e){System.err.println(e);}
      System.out.println( answer );

      MyRuunableFuture future2 = new MyRuunableFuture();
      answer = -1;
      while( ! future2.isDone() ) // poll the future
      {
         //System.out.print(".");
      }
      System.out.println( future2.result ); // get the computed value
   }
}
